
class Map:

	def __init__(s, nullelem):
		s.d_x = 0	# x coordinate displacement
		s.d_y = 0	# y coordinate displacement
		s.cells = []	# contents of the map
		s.nullelem = nullelem

	def get(s, x, y):
		if x < s.d_x or y < s.d_y: return s.nullelem
		x, y = x - s.d_x, y - s.d_y
		if y >= len(s.cells) or x >= len(s.cells[y]): return s.nullelem
		return s.cells[y][x]

	def set(s, x, y, contents):
		if y < s.d_y: s._extend_up(s.d_y - y)
		y = y - s.d_y
		if y >= len(s.cells):
			s.cells.extend(s._extension(y + 1 - len(s.cells)))
		if x < s.d_x: s._extend_left(s.d_x - x)
		x = x - s.d_x
		if x >= len(s.cells[y]):
			s.cells[y].extend([s.nullelem] * (x+1-len(s.cells[y])))
		s.cells[y][x] = contents

	def row_iterate(s, f):
		for row in s.cells: f(row)

	def _extension(s, length):
		return [[] for i in range(length)]

	def _extend_up(s, rows):
		s.cells = s._extension(rows) + s.cells
		s.d_y = s.d_y - rows

	def _extend_left(s, columns):
		ext = [s.nullelem] * columns
		for i in range(len(s.cells)):
			s.cells[i] = ext + s.cells[i]
		s.d_x = s.d_x - columns

	def bounds(s):
		raw_width = max([0] + map(len, s.cells))
		raw_height = len(s.cells)
		return s.d_x, s.d_y, raw_width + s.d_x, raw_height + s.d_y

def test_Map():
	m = Map(0)
	assert m.bounds() == (0,0,0,0)
	assert m.get(5,7) == 0
	m.set(5,7,2)
	assert m.get(5,7) == 2
	assert m.get(5,5) == 0
	assert m.get(7,5) == 0
	assert m.bounds() == (0,0,6,8)
	m.set(-3,-1,3)
	assert m.get(-3,-1) == 3
	assert m.get(5,7) == 2
	assert m.get(0,0) == 0
	assert m.bounds() == (-3,-1,6,8)
	print "test_Map: success"

if __name__ == '__main__':
	test_Map()
